package decisionanalysis;

import java.io.StreamTokenizer;
import java.io.IOException;

/**
 * A decision tree decision node.
 *
 * @author H&aring;kan L. Younes
 */
final class DTDecisionNode extends DTNode
{
    /**
     * The descendants of this node.
     */
    private DTNode[] alternatives;
    
    
    /**
     * Reads a decision node from the given source.
     *
     * @param st
     *  source for node description.
     * @param tree
     *  decision tree that the node belongs to.
     * @return the read node.
     * @exception IOException
     *  if an I/O error occurs.
     */
    protected static DTNode readFrom(StreamTokenizer st, DecisionTree tree)
	 throws IOException
    {
	if (st.nextToken() != StreamTokenizer.TT_NUMBER)
	{
	    throw new IOException("illegal token: " + st);
	}
	int n = (int)st.nval;
	DTDecisionNode node = new DTDecisionNode(tree, n);
	for (int i = 0; i < n; i++)
	{
	    node.alternatives[i] = DTNode.readFrom(st, tree);
	}
	return node;
    }


    /**
     * Constructs a decision tree decision node.
     *
     * @param tree
     *  decision tree that the node belongs to.
     * @param numAlternatives
     *  number of alternatives.
     */
    public DTDecisionNode(DecisionTree tree, int numAlternatives)
    {
	super(tree);
	alternatives = new DTNode[numAlternatives];
    }


    /**
     * Returns the expected utility for this node.
     *
     * @return the expected utility for this node.
     */
    public double expectedUtility()
    {
	double eu = alternatives[0].expectedUtility();
	for (int i = 1; i < alternatives.length; i++)
	{
	    double tmpEU = alternatives[i].expectedUtility();
	    if (tmpEU > eu)
	    {
		eu = tmpEU;
	    }
	}
	return eu;
    }

    /**
     * Returns the decision that maximizes the expected utility.
     *
     * @return the decision that maximizes the expected utility.
     */
    public int getOptimalDecision()
    {
	double eu = alternatives[0].expectedUtility();
	int best = 0;
	for (int i = 1; i < alternatives.length; i++)
	{
	    double tmpEU = alternatives[i].expectedUtility();
	    if (tmpEU > eu)
	    {
		eu = tmpEU;
		best = i;
	    }
	}
	return best;
    }

    /**
     * Returns a string representation of this node.
     *
     * @param depth
     *  depth of node in decision tree.
     * @return a string representation of this node.
     */
    public String toString(int depth)
    {
	StringBuffer sb = new StringBuffer("DECISION_NODE ");
	sb.append(alternatives.length);
	for (int i = 0; i < alternatives.length; i++)
	{
	    sb.append('\n');
	    for (int j = 0; j <= depth; j++)
	    {
		sb.append('\t');
	    }
	    sb.append(alternatives[i].toString(depth + 1));
	}
	return sb.toString();
    }
}
