package decisionanalysis;

import java.io.StreamTokenizer;
import java.io.IOException;

/**
 * A decision tree terminal node.
 *
 * @author H&aring;kan L. Younes
 */
final class DTTerminalNode extends DTNode
{
    /**
     * Number of utility variables that have been used.
     */
    protected static int maxU = 0;


    /**
     * Index of utility of this node in the value array of the tree
     * that this node belongs to.
     */
    private int utilIndex;


    /**
     * Reads a terminal node from the given source.
     *
     * @param st
     *  source for node description.
     * @param tree
     *  decision tree that the node belongs to.
     * @return the read node.
     * @exception IOException
     *  if an I/O error occurs.
     */
    protected static DTNode readFrom(StreamTokenizer st, DecisionTree tree)
	 throws IOException
    {
	if (st.nextToken() != 'u')
	{
	    throw new IOException("illegal token: " + st);
	}
	if (st.nextToken() != StreamTokenizer.TT_NUMBER)
	{
	    throw new IOException("illegal token: " + st);
	}
	int u = (int)st.nval;
	if (u > maxU)
	{
	    maxU = u;
	}
	return new DTTerminalNode(tree, u - 1);
    }


    /**
     * Constructs a decision tree terminal node.
     *
     * @param tree
     *  decision tree that the node belongs to.
     * @param utilIndex
     *  index of utility variable to use for this node.
     */
    public DTTerminalNode(DecisionTree tree, int utilIndex)
    {
	super(tree);
	this.utilIndex = utilIndex;
    }


    /**
     * Returns the expected utility for this node.
     *
     * @return the expected utility for this node.
     */
    public double expectedUtility()
    {
	return tree.getUtility(utilIndex);
    }

    /**
     * Returns a string representation of this node.
     *
     * @param depth
     *  depth of node in decision tree.
     * @return a string representation of this node.
     */
    public String toString(int depth)
    {
	StringBuffer sb = new StringBuffer("TERMINAL_NODE u");
	sb.append(utilIndex + 1);
	return sb.toString();
    }
}
